%% 1 - Root of the analysis

rootName = '\Users\Brain\Documents\Blapla\Analysis_Pipeline\Videos\Pilotes';
cd(rootName);



%% 2- Videos for level one analysis (Single session)

VideoScreenName = ['\Users\Brain\Documents\Blapla\Analysis_Pipeline\Videos\Pilotes\',...
    'sourcedata/2021-01-06_sub21/2021-01-06 10-43-39_screen.mp4'];
VideoGhostName = ['\Users\Brain\Documents\Blapla\Analysis_Pipeline\Videos\Pilotes\',...
    'sourcedata/2021-01-06_sub21/2021-01-06 10-43-39_ghost.mp4'];


%% Extracting videos properties

vScreen = VideoReader(VideoScreenName);
vGhost = VideoReader(VideoGhostName);

nbFrame = round(vScreen.Duration*vScreen.FrameRate);
fs = vScreen.FrameRate;                         % Extracting number of frames
X = vScreen.Width;                                % Size of Images - x-axis
Y = vScreen.Height;                               % Size of Images - y-axis
T = vScreen.Duration;                             % Duration of video


%Displaying the features
fprintf(['Video features extracted:\n'...
    'Width = %d px\n',...
    'Height = %d px\n',...
    'Duration = %1.2f s\n',...
    'Sampling Rate = %1.2f\n',...
    'Number of Frames = %d\n\n',...
    ], X, Y, vGhost.Duration, fs, nbFrame)

%% First reading of the video (mainly gets the ttl pixel signal and transition between slides)

vScreen.currentTime = 0;
vGhost.currentTime = 0;

th = 232;

tracks = zeros(3, nbFrame);
ttl = zeros(3, nbFrame);
frame = 1;

diffImg = zeros(1,nbFrame);
memFrame = zeros(Y,X);

while hasFrame(vScreen)
   
    fprintf('Analysing frame: %d / %d - t = %1.2f \n', frame, nbFrame, frame/fs);
    
    frameScreen = readFrame(vScreen);
    frameGhost = readFrame(vGhost);
    
    diffImg(1,frame) = sum(abs(memFrame - double(frameScreen(:,:,3))), [1,2]);  % Computes the difference of intensity to detect transition in task
    memFrame = double(frameScreen(:,:,3));
    
    BW = frameGhost(:,:,1) > th;
    
    stats = regionprops(BW);                % Extract Binary spot properties
    
    if size(stats)
        [C1] = stats.Centroid;                  % Gets the centroid
    end
    
    tracks(:,frame) = [frame/fs, C1(1,:)];
    
    ttl(:,frame) = frameScreen(2,2,:);
    
    frame = frame+1;
    
end

temps = (1:frame)/fs;                   % Vector of time

%% Detect Transitions (detects changes of image intensity i.e. slide transition)

k = size(tracks');

clf

ind = find(diffImg>800000);                     % Aribitrary Threshold of intensity change 
dind = diff(ind)/fs;
mind = find(dind> 0.8);                         % Arbitrary threshold for distance between found changes
size(mind)


plot((1:k)/fs, diffImg(1:k));                                   % displays the transitions that were detected
hold on
scatter(temps(ind(mind+1)), diffImg(ind(mind+1)), '+')
ylim([0, 10000000])

% ChangesIdx = ind(mind+1)-1;

ChangesIdx = ind(mind+1);
nbChanges = size(ChangesIdx,2);

fprintf('We found %d transitions\n\n', nbChanges)


%% Selects the calibration Bit

% Gets the red ttl for tasks events

indG = find(ttl(1,:) > 240);
dind = find(diff(indG) >1);

% Gets the transition within the first task (=calibration)

ind = find(temps(ChangesIdx)>=temps(min(indG)));
mind = find(temps(ChangesIdx(ind))<temps(indG(dind(1))));

disp('There are as events:')
fprintf('Event #:%d\n',mind);

%% Extracting Command and Measured data for warping measured images

% Finding the dots

fprintf('Extracting targets...')

nbPts = 12;
posPts = zeros(2, nbPts);

for pt = 1:nbPts
    
    vScreen.CurrentTime = temps(ChangesIdx(ind(mind(pt))))+0.5;
    frame = readFrame(vScreen);
    
    BW = frame(:,:,2)<200;                                                           % Find the dot

    if sum(BW, [1,2])

        stats = regionprops(BW);                % Extract Binary spot properties
        [C1] = stats.Centroid;                  % Gets the centroid

    end
    
    posPts(:,pt) = [C1(1);C1(2)];
    
end

fprintf('OK\n\nGetting the measured calibration...\n\n')

reconstructedCMD = zeros(Y, X);
reconstructedMES = zeros(Y, X);

for i = 1:nbPts
    
    fprintf('Calibration Point: %d/%d \n', i, nbPts);
    
    currentTime = temps(ChangesIdx(ind(mind(i))))+0.95;
    set(vScreen, 'CurrentTime', currentTime);
    set(vGhost, 'CurrentTime', currentTime);
    imgScreen = readFrame(vScreen);
    imgGhost = readFrame(vGhost);

    BW = (imgScreen(:,:,3) < 100);

    ConnectedSets = bwconncomp(BW);
    stats = regionprops(ConnectedSets);
    [~, newIdx] = sort([stats.Area], 'descend');
    reconstructedCMD(ConnectedSets.PixelIdxList{newIdx(1)}) = 1;

    BW = (imgGhost(:,:,1) > 250);

    ConnectedSets = bwconncomp(BW);
    stats = regionprops(ConnectedSets);
    [~, newIdx] = sort([stats.Area], 'descend');
    reconstructedMES(ConnectedSets.PixelIdxList{newIdx(1)}) = 1;
    
end

fprintf('\nOK\n\n')


%% Computes the warping factor for realignment (needs operator for selecting corrected points)

% shows before registration

close all

imshowpair(reconstructedCMD,reconstructedMES);
title('Before registration')

fixedPoints = ginput(4);
movingPoints = ginput(4);

Rfixed = imref2d(size(reconstructedCMD));    

tform = fitgeotrans(movingPoints,fixedPoints,'affine');
RegisteredMES = imwarp(reconstructedMES,tform,'OutputView',Rfixed);                       % Transform recorded data to check validity of transform
figure

imshowpair(reconstructedCMD,RegisteredMES);
title('After registration')

%% Warp Ghost  

disp('Let''s warp stuffs')

% Strat - 2 for reading the movie (the true one)

vGhost = VideoReader(VideoGhostName);            % Initialise videos

tracksCorrected = zeros(nbFrame,3);

frame = 1

% Reading the video

while hasFrame(vGhost) && (frame/fs)< (vGhost.Duration-1)
    
    fprintf('Frame : %d / %d - %1.2f s \n', frame, nbFrame, frame/fs);
    
    frameGhost = readFrame(vGhost);
    RegisteredFrameGhost = imwarp(frameGhost,tform,'OutputView',Rfixed);  
    
    BW = RegisteredFrameGhost(:,:,1)>200;                                                           % Find the dot
    
    if sum(BW, [1,2])

        stats = regionprops(BW);                % Extract Binary spot properties
        [C1] = stats.Centroid;                  % Gets the centroid

    end
        
    tracksCorrected(frame,:) = [frame/fs, C1(1), C1(2)];                  % Memorize the postion 
    
    frame = frame+1;
     
end

frame = size(ttl,2)

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% RESULTS

a = split(VideoScreenName, 'sourcedata');
b = split(a{2}, '/');
saveFolder = fullfile(rootName, 'derivatives', b{2});

if ~exist(saveFolder)
    mkdir(saveFolder)
end

%% Task (finds the periods when subject is solving a task)
 
ind = [find(ttl(1,1:frame) > 240), frame];
dind = find(diff(ind) > 1);
mind = ind(dind);
fins = temps(mind);
finsTask = fins(2:end);
mind = ind(dind+1);
debs = temps(mind);
debsTask = debs(1:end-1);

nbTask = size(debsTask,2);


%% postTask (finds periods when subject watches intermediary results)
 
ind = [1,find(ttl(2,1:frame) > 240),frame];
dind = find(diff(ind) > 1);
mind = ind(dind);
fins = temps(mind);
finsPost = fins(2:end);
mind = ind(dind+1);
debs = temps(mind);
debsPost = debs(1:end-1);


%% Cross Time (finds the periods when subject looks at the cross)

a = (ttl(1,1:frame) < 200);
b = (ttl(1,1:frame) > 100);

ind = [1,find(a+b == 2),frame];
dind = find(diff(ind) > 1);
mind = ind(dind);
fins = temps(mind);
finsCross = fins(2:end);
mind = ind(dind+1);
debs = temps(mind);
debsCross = debs(1:end-1);

%% Loop of analysis 

for task = 1:nbTask

    %% Export (Building the output directories for the task, post task and cross events)
    
    taskDir = fullfile(saveFolder, sprintf('%d_Task-%d', task, task));
    if~exist(taskDir, 'file')
        mkdir(taskDir)
    end
    
    postDir = fullfile(saveFolder, sprintf('%d_Post-%d', task, task));
    if~exist(postDir, 'file')
        mkdir(postDir)
    end
    
    crossDir = fullfile(saveFolder, sprintf('%d_cross-%d', task, task));
    if~exist(crossDir, 'file')
        mkdir(crossDir)
    end 
    
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% TASK
    %

    deb = debsTask(task);              % Initial time of block in s
    fin = finsTask(task);              % End time of block in s

    fprintf('Analysing task: #%d from: %1.2fs to %1.2fs \n\n', task, deb, fin)    
    
    set(vScreen, 'CurrentTime', (deb+fin)/2);

    %% Extracting Targets (gets set of pixel in the screen image -> operator + digits)

    fprintf('Extracting targets...\n')
    
    frame = readFrame(vScreen);

    BW = frame(:,:,1) == 0;

    ConnectedSets = bwconncomp(BW);
    nbSpots = ConnectedSets.NumObjects;
    a = ConnectedSets.ImageSize;
    Y = a(1);
    X = a(2);

    posPts = zeros(2, nbSpots);
    k = 1;

    for spot = 1:nbSpots

        fprintf('Spot: %d/%d - k=%d\nSize: %d\n\n', spot, nbSpots, k, size(ConnectedSets.PixelIdxList{spot}, 1))

        if size(ConnectedSets.PixelIdxList{spot},1)>= 80

            spotBW = zeros(Y, X);
            spotBW(ConnectedSets.PixelIdxList{spot})=1;
            stats = regionprops(spotBW);

            C1 = stats.Centroid;
            posPts(:,k) = [C1(1), C1(2)];
            k = k+1;

        end
    end

    posPts(:,k:end) = [];
    nbSpots = size(posPts,2);

    fprintf('We found %d targets\n\n', nbSpots)
    
    %% Getting the trace


    ind = find(temps>deb);
    mind = find(temps(ind)<fin);

    subTemps = temps(ind(mind));
    subX = tracksCorrected(ind(mind),2);
    subY = tracksCorrected(ind(mind),3);
    subDur = size(subTemps, 2);

    %% DISPLAY
    
    % Targets

    close all
    clf

    imshow(frame)
    hold on
    scatter(posPts(1,:), posPts(2, :), '+b')
    
    saveas(gcf, fullfile(taskDir, '1_targets.png'))


    %% Simple trajectory

    imshow(frame)
    hold on
    plot(subX, subY)
    
    saveas(gcf, fullfile(taskDir, '2_trajectory.png'))

    %% Simple Heatmap

    X = vScreen.Width;
    Y = vScreen.Height;

    BW = zeros(X, Y);

    for i = 1:subDur

        BW(round(subX(i)), round(subY(i))) = BW(round(subX(i)), round(subY(i))) + 1;

    end

    BW(1,:) = 0;
    BW(end,:) = 0;
    BW(:,1) = 0;
    BW(:,end) = 0;

    L = 500;
    S = 50;

    GaussKernel =  fspecial('gaussian', L,S);
    GaussKernel = GaussKernel/max(GaussKernel, [], [1,2]);
    heatMap = conv2(BW, GaussKernel, 'same')/fs;

    clf 

    imshow(frame);
    hold on
    imagesc(heatMap', 'AlphaData', 0.5*heatMap')
    colormap('hot')
    saveas(gcf,fullfile(taskDir,'3_heatmap.png'))
    
    
    %% Trajectory and heatmap


    clf

    ax1 = axes();
    ax2 = axes();

    set(gcf,'CurrentAxes',ax1)

    imagesc(frame)
    hold on
    imagesc(heatMap', 'AlphaData', 0.5*heatMap')
    xticks([])
    yticks([])

    set(gcf,'CurrentAxes',ax2)

    patch(ax2, [subX; flipud(subX)], [subY;flipud(subY)], [1:size(subX,1), fliplr(1:size(subX,1))],...
        'EdgeColor','interp', 'lineWidth', 1,'MarkerFaceColor','flat');
    colormap(ax2,'jet');
    set(ax2, 'YDir', 'reverse')

    colormap(ax1, 'hot');

    linkaxes([ax1, ax2], 'xy')
    ax2.Visible = 'off';
    set(gcf, 'Unit', 'normalized', 'Position', [0,0,1,1])
    
    saveas(gcf,fullfile(taskDir, '4_TrajAndHeatmap.png'))
    
    
    %% Word Analysis
    
    spotDist = zeros(nbSpots, subDur);

    for spot = 1:nbSpots

        a = sqrt(sum(([subX, subY]-[posPts(:,spot)]').^2,2));
        spotDist(spot,:) = a<40;

    end

    clf

    subplot(2,1,1)

    imagesc(frame)
    axis equal
    xticks([])
    yticks([])
    hold on

    for spot = 1:nbSpots
        text(posPts(1,spot), posPts(2,spot)+80, sprintf('%d', spot),...
            'color', 'red',...
            'HorizontalAlignment', 'center',...
            'FontSize', 5)
    end

    ax2 = subplot(2,1,2)

    imagesc(ax2,spotDist)
    colormap('gray')
    for i = 2:nbSpots
        yline(i-0.5, 'w:')
    end
    ylim([0.5, nbSpots+0.5])
    yticks((1:nbSpots))
    xlabel('Time (s)', 'Interpreter', 'Latex')
    ylabel('Target ID', 'Interpreter', 'Latex')

    set(ax2, 'TickLabelInterpreter', 'Latex')
    
    saveas(gcf,fullfile(taskDir, '5_Words.png'));

    %% Trinary Word Analysis
    
    xLeft = 605;
    xRight = 682;

    a = temps>deb;
    b = temps<fin;
    
    ind = find(a+b==2);

    trinaryWord = zeros(3,subDur);
    a = tracksCorrected(ind,2) < xLeft;
    b = tracksCorrected(ind,2) > xRight;
    trinaryWord(1,:) = a;
    trinaryWord(2,:) = a+b==0;
    trinaryWord(3,:) = b;
    
    clf

    subplot(2,1,1)

    imagesc(frame)
    axis equal
    xticks([])
    yticks([])
    hold on
    xline(xLeft, ':k')
    xline(xRight, ':k')

    ax2 = subplot(2,1,2)
    
    imagesc(ax2,trinaryWord)
    colormap('gray')
    for i = 2:3
        yline(i-0.5, 'w:')
    end
    ylim([0.5, 3+0.5])
    yticks((1:3))
    yticklabels({'L', 'O', 'R'})
    xlabel('Time (s)', 'Interpreter', 'Latex')
    ylabel('Target ID', 'Interpreter', 'Latex')

    set(ax2, 'TickLabelInterpreter', 'Latex')
    
    saveas(gcf,fullfile(taskDir, '6_TrinaryWords.png'));
    
    
    %% Saving the task Object summarizing the analysis    
    
    taskObj = struct();
    
    taskObj.glob = struct();
    taskObj.glob.deb = deb;
    taskObj.glob.fin = fin;
    taskObj.glob.frame = frame;
    
    taskObj.spot = struct();
    taskObj.spot.posPts = posPts;
    taskObj.spot.nbSpots = nbSpots;
    
    taskObj.trace = struct();
    taskObj.trace.subTemps = subTemps;
    taskObj.trace.subX = subX;
    taskObj.trace.subY = subY;
    
    taskObj.heatMap = heatMap;
    
    taskObj.spotDist = spotDist;
    taskObj.trinaryWord = trinaryWord;
    
    save(fullfile(taskDir, 'task.mat'), 'taskObj')
    

    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Post TASK
    %

    deb = debsPost(task);              % Initial time of block in s
    fin = finsPost(task);              % End time of block in s

    fprintf('Analysing Post: #%d from: %1.2fs to %1.2fs \n\n', task, deb, fin)    
    
    set(vScreen, 'CurrentTime', (deb+fin)/2);

    %% Extracting Targets (gets set of pixel in the screen image -> digits)

    fprintf('Extracting targets...\n')
    
    frame = readFrame(vScreen);

    BW = frame(:,:,1) <= 50;

    % BW = reconstructedCMD;

    ConnectedSets = bwconncomp(BW);
    nbSpots = ConnectedSets.NumObjects;
    a = ConnectedSets.ImageSize;
    Y = a(1);
    X = a(2);

    posPts = zeros(2, nbSpots);
    k = 1;

    for spot = 1:nbSpots

        fprintf('Spot: %d/%d - k=%d\nSize: %d\n\n', spot, nbSpots, k, size(ConnectedSets.PixelIdxList{spot}, 1))

        if size(ConnectedSets.PixelIdxList{spot},1)>= 80

            spotBW = zeros(Y, X);
            spotBW(ConnectedSets.PixelIdxList{spot})=1;
            stats = regionprops(spotBW);

            C1 = stats.Centroid;
            posPts(:,k) = [C1(1), C1(2)];
            k = k+1;

        end
    end

    posPts(:,k:end) = [];
    nbSpots = size(posPts,2);

    fprintf('We found %d targets\n\n', nbSpots)

    %% Extracting mega targets (clustering neighbour targets to extract multi digits targets)

    dists = zeros(3,(nbSpots*(nbSpots-1))/2);

    k = 1;

    % Computing distance between nodes
    
    for spot1 = 1:nbSpots

        for spot2 = 1:spot1-1

            dists(1,k) = spot1;
            dists(2,k) = spot2;
            dists(3,k) = sqrt((posPts(1,spot1)-posPts(1,spot2)).^2+(posPts(2,spot1)-posPts(2,spot2)).^2);
            k = k+1;


        end

    end 

    % computing the clusters of two elements and add them
    
    ind = find(dists(3,:)<55);
    nbClustElt = size(ind,2);
    clustList = zeros(2,nbClustElt);
    clustMem = [];
    
    for clust = 1:nbClustElt
        
        clustList(1,clust) = mean([posPts(1,dists(1, ind(clust))), posPts(1,dists(2, ind(clust)))]);
        clustList(2,clust) = mean([posPts(2,dists(1, ind(clust))), posPts(2,dists(2, ind(clust)))]);
        clustMem = [clustMem, dists(1, ind(clust)), dists(2, ind(clust))];        
        
    end
    
    % Adds single elements cluster
    
    for spot = 1:nbSpots
        
        disp(spot)
        
        if ~ismember(spot, clustMem)
            
            clustList = [clustList, posPts(:,spot)];
            
        end
    end
    
    nbClusters = size(clustList, 2);    

    %% Getting the trace

    ind = find(temps>deb);
    mind = find(temps(ind)<fin);

    subTemps = temps(ind(mind));
    subX = tracksCorrected(ind(mind),2);
    subY = tracksCorrected(ind(mind),3);
    subDur = size(subTemps, 2);

    %% DISPLAY
    
    % Targets

    close all
    clf

    imshow(frame)
    hold on
    scatter(posPts(1,:), posPts(2, :), '+b')
    
    saveas(gcf, fullfile(postDir, '1_targets.png'))

    % Simple trajectory

    imshow(frame)
    hold on
    plot(subX, subY)
    
    saveas(gcf, fullfile(postDir, '2_trajectory.png'))

    % Simple Heatmap

    X = vScreen.Width;
    Y = vScreen.Height;

    BW = zeros(X, Y);

    for i = 1:subDur

        BW(round(subX(i)), round(subY(i))) = BW(round(subX(i)), round(subY(i))) + 1;

    end

    BW(1,:) = 0;
    BW(end,:) = 0;
    BW(:,1) = 0;
    BW(:,end) = 0;

    L = 500;
    S = 50;

    GaussKernel =  fspecial('gaussian', L,S);
    GaussKernel = GaussKernel/max(GaussKernel, [], [1,2]);
    heatMap = conv2(BW, GaussKernel, 'same')/fs;

    clf 

    imshow(frame);
    hold on
    imagesc(heatMap', 'AlphaData', 0.5*heatMap')
    colormap('hot')
    saveas(gcf,fullfile(postDir,'3_heatmap.png'))
    
    
    %% Heatmap and trajectories

    clf

    ax1 = axes();
    ax2 = axes();

    set(gcf,'CurrentAxes',ax1)

    imagesc(frame)
    hold on
    imagesc(heatMap', 'AlphaData', 0.5*heatMap')
    xticks([])
    yticks([])

    set(gcf,'CurrentAxes',ax2)

    patch(ax2, [subX; flipud(subX)], [subY;flipud(subY)], [1:size(subX,1), fliplr(1:size(subX,1))],...
        'EdgeColor','interp', 'lineWidth', 1,'MarkerFaceColor','flat');
    colormap(ax2,'jet');
    set(ax2, 'YDir', 'reverse')

    colormap(ax1, 'hot');

    linkaxes([ax1, ax2], 'xy')
    ax2.Visible = 'off';
    set(gcf, 'Unit', 'normalized', 'Position', [0,0,1,1])
    
    saveas(gcf,fullfile(postDir, '4_TrajAndHeatmap.png'))
    
    %% Word Analysis
    
    spotDist = zeros(nbSpots, subDur);

    for spot = 1:nbSpots

        a = sqrt(sum(([subX, subY]-[posPts(:,spot)]').^2,2));
        spotDist(spot,:) = a<20;

    end

    clf

    subplot(2,1,1)

    imagesc(frame)
    axis equal
    xticks([])
    yticks([])
    hold on

    for spot = 1:nbSpots
        text(posPts(1,spot), posPts(2,spot)+80, sprintf('%d', spot),...
            'color', 'red',...
            'HorizontalAlignment', 'center',...
            'FontSize', 5)
    end

    ax2 = subplot(2,1,2)

    imagesc(ax2,spotDist)
    colormap('gray')
    for i = 2:nbSpots
        yline(i-0.5, 'w:')
    end
    ylim([0.5, nbSpots+0.5])
    yticks((1:nbSpots))
    xlabel('Time (s)', 'Interpreter', 'Latex')
    ylabel('Target ID', 'Interpreter', 'Latex')

    set(ax2, 'TickLabelInterpreter', 'Latex')
    
    saveas(gcf,fullfile(postDir, '5_Words.png'));

    %% Cluster Word Analysis
    
    spotDist = zeros(nbClusters, subDur);

    % Reordering Clusters
    
    a = clustList - [X/2; Y/2];
    alphas = atan(a(1,:)./a(2,:));
    [~, Idx] = sort(alphas);
    
    clustList = clustList(:,Idx);

    for spot = 1:nbClusters

        a = sqrt(sum(([subX, subY]-[clustList(:,spot)]').^2,2));
        spotDist(spot,:) = a<50;

    end

    clf

    subplot(2,1,1)

    imagesc(frame)
    axis equal
    xticks([])
    yticks([])
    hold on
    viscircles(clustList', 50*ones(1, nbClusters));

    for spot = 1:nbClusters
        text(clustList(1,spot), clustList(2,spot)+80, sprintf('%d', spot),...
            'color', 'blue',...
            'HorizontalAlignment', 'center',...
            'FontSize', 5)
    end

    ax2 = subplot(2,1,2)

    imagesc(ax2,spotDist)
    colormap('gray')
    for i = 2:nbClusters
        yline(i-0.5, 'w:')
    end
    ylim([0.5, nbClusters+0.5])
    yticks((1:nbClusters))
    xlabel('Time (s)', 'Interpreter', 'Latex')
    ylabel('Target ID', 'Interpreter', 'Latex')

    set(ax2, 'TickLabelInterpreter', 'Latex')
    
    %
    
    saveas(gcf,fullfile(postDir, '6_ClusterWords.png'));

    
    %% Saving the post Object summarizing the analysis 

    postObj = struct();
    
    postObj.glob = struct();
    postObj.glob.deb = deb;
    postObj.glob.fin = fin;
    postObj.glob.frame = frame;
    
    postObj.spot = struct();
    postObj.spot.posPts = posPts;
    postObj.spot.nbSpots = nbSpots;
    
    postObj.clust = struct();
    postObj.clust.nbClusters = nbClusters;
    postObj.clust.clustList = clustList;
    
    postObj.trace = struct();
    postObj.trace.subTemps = subTemps;
    postObj.trace.subX = subX;
    postObj.trace.subY = subY;
    
    postObj.heatMap = heatMap;
    
    postObj.spotDist = spotDist
    
    save(fullfile(postDir, 'post.mat'), 'postObj')
    

    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% CROSS
    %

    deb = debsCross(task);              % Initial time of block in s
    fin = finsCross(task);              % End time of block in s

    fprintf('Analysing task: #%d from: %1.2fs to %1.2fs \n\n', task, deb, fin)    
    
    set(vScreen, 'CurrentTime', (deb+fin)/2);
    
    % Extracting Target

    fprintf('Extracting target...\n')
    
    frame = readFrame(vScreen);


    % Superposition target and observation
    
    set(vGhost, 'CurrentTime', fin);
    frameGhost = readFrame(vGhost);
    
    RegisteredFrameGhost = imwarp(frameGhost,tform,'OutputView',Rfixed);  
    
    clf
    set(gcf, 'Unit', 'pixel', 'Position', [10,10,1000,500])
    
    subplot(1,2,1)
    imshowpair(frame, frameGhost)
    title([sprintf('Cross : %d', task),newline,sprintf('Before Registration')], 'Interpreter', 'Latex')
    subplot(1,2,2)
    imshowpair(frame, RegisteredFrameGhost)
    title([sprintf('Cross : %d', task),newline,sprintf('After Registration')], 'Interpreter', 'Latex')
    
    saveas(gcf, fullfile(crossDir, '1_calibrationCheck.png'))
        
    
    % distance from target
    
    a = (frame(:,:,1)) > 200;
    b = (frame(:,:,2)) < 100;
    
    BW = a+b == 2;

    ConnectedSets = bwconncomp(BW);
   
    spotBW = zeros(Y,X);
    spotBW(ConnectedSets.PixelIdxList{1})=1;
    stats = regionprops(spotBW);
    C1 = stats.Centroid;
    
    % Display
    
    
    a = temps > deb;
    b = temps < fin;
    ind = find(a+b == 2);
    
    dist = sqrt((tracksCorrected(ind, 2)-C1(1)).^2+(tracksCorrected(ind, 3)-C1(2)).^2);
    
    clf
    set(gcf, 'Unit', 'pixel', 'Position', [10,10,1000,500])
    
    plot(temps(ind), dist)
    xlabel('Time (s)', 'Interpreter', 'latex')
    ylabel('Distance (px)', 'Interpreter', 'latex')
    set(gca, 'TickLabelInterpreter', 'latex')
    title([sprintf('Cross : %d', task), newline, 'Distance from target'], 'Interpreter', 'latex')
    
    saveas(gcf, fullfile(crossDir, '2_calibrationDistance.png'))

    %% Saving the cross Object summarizing the analysis 

    crossObj = struct();
    
    crossObj.glob = struct();
    crossObj.glob.deb = deb;
    crossObj.glob.fin = fin;
    crossObj.glob.frame = frame;
    
    crossObj.spot = struct();
    crossObj.spot.posPts = posPts;
    crossObj.spot.nbSpots = nbSpots;
    
    crossObj.trace = struct();
    crossObj.trace.subTemps = subTemps;
    crossObj.trace.subX = subX;
    crossObj.trace.subY = subY;
    
    crossObj.C1 = C1;
    
    crossObj.dist = dist;
       
    save(fullfile(crossDir, 'cross.mat'), 'crossObj')
    
end
    

%% SavingParameters

paramDir = fullfile(saveFolder, 'Params');
mkdir(paramDir);

% Pre Registration


imshowpair(reconstructedCMD,reconstructedMES);
title('Before registration', 'Interpreter', 'latex', 'FontSize', 30)

set(gca, 'Unit', 'normalized', 'Position', [0.1,0.1,0.8,0.8])
set(gcf, 'Unit', 'normalized', 'Position', [0,0,1,1])

saveas(gcf, fullfile(paramDir, '1_preRegistration.png'))

% Post Registration

clf

imshowpair(reconstructedCMD,RegisteredMES);
title('After registration')


set(gca, 'Unit', 'normalized', 'Position', [0.1,0.1,0.8,0.8])
set(gcf, 'Unit', 'normalized', 'Position', [0,0,1,1])

saveas(gcf, fullfile(paramDir, '2_postRegistration.png'))

%

save(fullfile(paramDir, '3_tform.mat'), 'tform');


