clear variables

addpath('C:\Users\Admin\Documents\MATLAB\Toolboxes');       % Adding libraries

fprintf('The library is added\n\n');


%% 1 - Parameters: Roots of the analysis, Absolute params

rootName = 'D:\Blapla\Analysis_Pipeline\Videos\Pilotes\sourcedata';                  % root where videos are (sourcedata)
derivativeRoot = 'D:\Blapla\Analysis_Pipeline\Videos\Pilotes\derivatives';  % root of Pipeline3 outputs fo metadata (beginnings and ends) 
cd(rootName);

fprintf('Root has moved to:\n%s\n\nMetadata will be loaded from:\n%s\n\n', rootName, derivativeRoot);

BWth = 230;             % Binarisation threshold of red spot
nbPtTh = 150;           % Threshold of nbPoint derivatives underneath no displacement is assumed
radius = 16.5;            % radius of spot
vTh = 600;

saveDir = 'D:\Blapla\Analysis_Pipeline\Videos\Pilotes\derivatives_v2\';
if ~exist(saveDir, 'file')
    mkdir(saveDir);
end

%% 2 - Filtering for files of interest

% use of regex to get videos of interest
%listDir = regexpdir(rootName, '.*sub(1[4-9]|2[0-3]).*ghost.*$');
listDir = regexpdir(rootName, '.*(sub25).*ghost.*$');
nbFiles = size(listDir, 1);
fprintf('We found %d files\n\n', nbFiles);

%% 3 - Loop through the videos

for file = 1:nbFiles
    
   %%
    
   videoName = listDir{file};
   
   % Features extraction
   
   a = split(videoName, filesep);
   b = split(videoName, 'ghost');
   videoScreenName = [b{1}, 'screen.mp4'];
   
   b = split(a{end-1}, '_');
   Datum = b{1};
   subjectName = b{2};
   
   % filtering for event
   
   % regex again
   
   % Change task if you want to check cross or Post you will have to change the object type to PostObj further down
   
   regexFilt = sprintf('.*%s.*%s.*Post.*.mat$', Datum, subjectName);
   eventDir = regexpdir(derivativeRoot, regexFilt);
   
   % Gets the number of task for this subject
   nbEvents = size(eventDir, 1);
   
   fprintf('We found %d events\n\n', nbEvents);
   
   % Goes through all tasks (or cross or post) called events
   
   %%
   for event = 1:nbEvents
       
       %%
       videoName = listDir{file};
       
       fprintf('Event: %d/%d\n%s\n\n', event, nbEvents, eventDir{event});
       load(eventDir{event}, '-mat');
       
       % Gets Features from event
       
       % relative path from derivatives to event file
       a = split(eventDir{event}, derivativeRoot);
       suffix = a{2};
       
       % To be changed if you want to use it for cross or post
       obj = postObj;
       
       %% Actual re-analysis
       
       % reading the video getting the noice properties
       vGhost = VideoReader(videoName);
       fs = 1/vGhost.FrameRate;
       frame = readFrame(vGhost);

       % Expected duration of event and size of image
        T = size(obj.spotDist,2);    % obj.spotDist pour Task et Post, obj.dist pour Cross
        temps = (1:T)/fs;
        [X, Y] = size(frame, (1:2));

        
        % Geting to the starting point
        vGhost.CurrentTime = obj.glob.deb;
        % reading first image
        mem = readFrame(vGhost);
        % throsholding the red blob
        mem = mem(:,:,1)>BWth;
        
        k = 1;
        stats = regionprops(mem);                % Extract Binary spot properties
        
        % Initiates spot position
        if size(stats)
            [C1] = stats.Centroid;                  % Gets the centroid
            x = C1(1);
            y = C1(2);
        else
            x = obj.trace.subX(1);
            y = obj.trace.subY(1);
        end

        % Initiates cursor estimates
        cursor1 = zeros(2,T);
        cursor2 = zeros(2,T);
        
         a = split(videoName,'ghost');
         videoName = strcat(a{1},'screen.mp4');
   
        vScreen = VideoReader(videoScreenName);
        nbFrame = round(vScreen.Duration*vScreen.FrameRate);
        
        tracksCorrected = zeros(T,3);

        % counter for while loop
        k = 1;
        
        
        %%
       
        while vGhost.CurrentTime < postObj.glob.fin

            %%

            fprintf('Frame: %d - ', k);

            % read frame
            frame = readFrame(vGhost);
            
            % gets the blob
            a = frame(:,:,1)>BWth;
            
%             % Old School
%             
%             stats = regionprops(a);                % Extract Binary spot properties
% 
%             if size(stats)
%                 [C1] = stats.Centroid;                  % Gets the centroid
%                 x1 = C1(1);
%                 y1 = C1(2);
%             end
% 
%             % first estimate (old school)
%             cursor1(:,k) = [x1;y1];

            % Finds pixel derivative (the pixels that appeared in the new frame)
            b = (a-mem == 1);

            % Displays the number of new pixels
            fprintf('%d\n', sum(b, [1,2]))

            % Updates the memory of frame for next evaluation of derivative
            mem = a;

            % If there were new pixels
            if find(b==1)

                % Gets the convex silhouette of new pixels
                [i,j] = ind2sub([X, Y], find(bwconvhull(b)));
                % finds the furtherthemost point of new pixels silouhette
                dist = sqrt((i-y).^2+(j-x).^2);
                [~,I] = sort(dist);

                % Gets the center of estimated spot based on spot
                % deformation (Thales theorem)
                ratio = radius/dist(I(end));
                x2 = j(I(end))-(j(I(end))-x)*ratio;
                y2 = i(I(end))-(i(I(end))-y)*ratio; 
                
                % New estimate
                
                x = x2;
                y = y2;

            end
            
%             tracksCorrected(frame,:) = [frame/fs, C1(1), C1(2)];
            tracksCorrected(k,:) = [k/fs,x,y];

% if no new pixel position is assumed not to change (loss of blob or no movement)

            cursor2(:,k) = [x;y];

            k = k+1;
            
        end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        % Old School
%         
%         dx = diff(obj.trace.subX);
%         dy = diff(obj.trace.subY);
%         speed0 = sqrt(dx.^2 + dy.^2)*30;
%         
%         dx = diff(cursor1(1,:));
%         dy = diff(cursor1(1,:));
%         speed1 = sqrt(dx.^2 + dy.^2)*30;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
                
        %% Sharp trace

        dx = diff(cursor2(1,:));
        dy = diff(cursor2(1,:));
        speed2 = sqrt(dx.^2 + dy.^2)*30;
        
        obj.sharpTrace = struct();
        obj.sharpTrace.cursor = cursor2;
        obj.sharpTrace.speed = speed2;
        
        % Gets the first datapoint of saccade
        
%         
        ind = (speed2<vTh);
        % removes saccade at the borders
        ind(1:2) = 1;
        ind(end-1:end) = 1;
        mind = ind-circshift(ind, -1);
        
        debs = find(mind == 1)+1;
        
        % Gets the last datapoint of saccade
        
        fins = find(mind == -1);
        
        % If saccade is on the last point
%         
%         if mind(1) == -1
%             
%             fins(1) = [];
%             debs(end) = [];
%             
%         end
        
%         
%         ind = (speed2<vTh);
%         mind = ind-circshift(ind, -1);
%         debs = find((ind+abs(mind) == 2))+1;
%         
%         % Gets the last datapoint of saccade
%         
%         ind = (speed2<vTh);
%         mind = ind-circshift(ind, +1);
%         fins = find((ind+abs(mind) == 2))-1;
        
        Centers = permute(cat(3, cursor2(:, max(1, debs-1)), cursor2(:, min(T,fins+1))), [2,1,3]); % nbSaccade, X-Y, deb-fin
        Dists = sqrt(sum((Centers(:,:,2)-Centers(:,:,1)).^2, 2));
        Durations = fins-debs+1;
        
        % Stores saccades properties
        
        obj.sharpTrace.saccade = struct();
        obj.sharpTrace.saccade.debs = debs;
        obj.sharpTrace.saccade.fins = fins;
        obj.sharpTrace.saccade.Centers = Centers;
        obj.sharpTrace.saccade.Dists = Dists;
        obj.sharpTrace.saccade.Durations = Durations;
        
        
        nbSaccade = size(debs, 2);
        
        maxSpeed = zeros(1, nbSaccade);
        for saccade = 1:nbSaccade
            if obj.sharpTrace.saccade.fins(saccade)<size(speed2,2)
                maxSpeed(saccade) = max(speed2(obj.sharpTrace.saccade.debs(saccade):obj.sharpTrace.saccade.fins(saccade)));
            end
        end
        
        obj.sharpTrace.saccade.maxSpeed = maxSpeed;
        
        
        % Fixation
        
        a = debs;
        debs = fins(1:end-1);
        fins = a(2:end);
        Centers =  cursor2(:, round((debs+fins)/2));   % nbSaccade, X-Y
        Durations = fins-debs+1;
        
        % stores fixation properties
        
        obj.sharpTrace.fixation = struct();
        obj.sharpTrace.fixation.debs = debs;
        obj.sharpTrace.fixation.fins = fins;
        obj.sharpTrace.fixation.Centers = Centers;
        obj.sharpTrace.fixation.Dists = Dists;
        obj.sharpTrace.fixation.Durations = Durations;

        %% Smoothed trace

        FiltWindow = 5;
        filtOrder = 3;

        % apply Savitzky-Golay filter to smoothen data
        
        cursor1x = sgolayfilt(cursor2(1,:),filtOrder, FiltWindow);
        cursor1y = sgolayfilt(cursor2(2,:), filtOrder, FiltWindow);
        
        dx = diff(cursor1x(1,:));
        dy = diff(cursor1y(1,:));
        speed2 = sqrt(dx.^2 + dy.^2)*30;
        
        obj.smoothTrace = struct();
        cursor1 = [cursor1x;cursor1y];
        obj.smoothTrace.cursor = cursor1;
        obj.smoothTrace.speed = speed2;
        
        % Gets the first datapoint of saccade
        
%         
        ind = (speed2<vTh);
        % removes saccade at the borders
        ind(1:2) = 1;
        ind(end-1:end) = 1;
        mind = ind-circshift(ind, -1);
        
        debs = find(mind == 1)+1;
        
        % Gets the last datapoint of saccade
        
        fins = find(mind == -1);
        
        % If saccade is on the last point
%         
%         if mind(1) == -1
%             
%             fins(1) = [];
%             debs(end) = [];
%             
%         end
        
%         
%         ind = (speed1<vTh);
%         mind = ind-circshift(ind, -1);
%         debs = find((ind+abs(mind) == 2))+1;
%         
%         % Gets the last datapoint of saccade
%         
%         ind = (speed1<vTh);
%         mind = ind-circshift(ind, +1);
%         fins = find((ind+abs(mind) == 2))-1;
        
        Centers = permute(cat(3, cursor1(:, max(1, debs-1)), cursor1(:, min(T,fins+1))), [2,1,3]); % nbSaccade, X-Y, deb-fin
        Dists = sqrt(sum((Centers(:,:,2)-Centers(:,:,1)).^2, 2));
        Durations = fins-debs+1;
        
        % Stores saccades properties
        
        obj.smoothTrace.saccade = struct();
        obj.smoothTrace.saccade.debs = debs;
        obj.smoothTrace.saccade.fins = fins;
        obj.smoothTrace.saccade.Centers = Centers;
        obj.smoothTrace.saccade.Dists = Dists;
        obj.smoothTrace.saccade.Durations = Durations;
        
        nbSaccade = size(obj.smoothTrace.saccade.debs, 2);
        
        maxSpeed = zeros(1, nbSaccade);
        for saccade = 1:nbSaccade
            if obj.smoothTrace.saccade.fins(saccade)<size(speed2,2)
                maxSpeed(saccade) = max(speed2(obj.smoothTrace.saccade.debs(saccade):obj.smoothTrace.saccade.fins(saccade)));
            end
        end

        
        obj.smoothTrace.saccade = struct();
        obj.smoothTrace.saccade.debs = debs;
        obj.smoothTrace.saccade.fins = fins;
        obj.smoothTrace.saccade.Centers = Centers;
        obj.smoothTrace.saccade.Dists = Dists;
        obj.smoothTrace.saccade.Durations = Durations;
        obj.smoothTrace.saccade.maxSpeed = maxSpeed;
        
        
        % Fixation
        
        a = debs;
        debs = fins(1:end-1);
        fins = a(2:end);
        Centers =  cursor2(:, round((debs+fins)/2));   % nbSaccade, X-Y
        Durations = fins-debs+1;
        
        % stores fixation properties
        
        obj.smoothTrace.fixation = struct();
        obj.smoothTrace.fixation.debs = debs;
        obj.smoothTrace.fixation.fins = fins;
        obj.smoothTrace.fixation.Centers = Centers;
        obj.smoothTrace.fixation.Dists = Dists;
        obj.smoothTrace.fixation.Durations = Durations;
        
        % Saving Object
                
        postObj = obj;
        
        
        % To be changed with type of event
        a = split(suffix, 'post.mat');
        if ~exist(fullfile(saveDir, a{1}), 'file')
            mkdir(fullfile(saveDir, a{1}))
        end
        
        save(fullfile(saveDir, suffix), 'postObj');
       
        
    %% DISPLAY
%    
%     a = split(videoName,'ghost');
%     videoName = strcat(a{1},'screen.mp4');
%    
    vScreen = VideoReader(videoScreenName);
    
    deb = obj.glob.deb;              % Initial time of block in s
    fin = obj.glob.fin;              % End time of block in s

    set(vScreen, 'CurrentTime', (deb+fin)/2);
    
    frame = readFrame(vScreen);
   
    b = split(suffix,'\');
    savePath = strcat(saveDir,b{2},'\',b{3},'\');
    
   %% 1 - Targets   ONLY FOR TASK AND POST
   
    close all
    clf

    imshow(frame)
    hold on
    scatter(obj.spot.posPts(1,:), obj.spot.posPts(2, :), '+b')
    
    saveas(gcf, fullfile(savePath, '1_targets.png'))

    % 2 - Simple trajectory    FOR TASK, POST AND CROSS

    subX = obj.sharpTrace.cursor(1,:);
    subY = obj.sharpTrace.cursor(2,:);
    
    
    imshow(frame)
    hold on
    plot(subX,subY)
    
    saveas(gcf, fullfile(savePath, '2_trajectory.png'))
    
    
    % 2bis - Trajectory comparison   FOR TASK, POST AND CROSS

    imshow(frame)
    hold on
    plot(subX,subY)
    hold on
    plot(obj.trace.subX,obj.trace.subY,'r')
    
    legend({'New extimator', 'Old estimator'}, 'Interpreter', 'latex')
    
    saveas(gcf, fullfile(savePath, '2bis_trajectoryComparison.png'))
    
    %% 3- Simple Heatmap   ONLY FOR TASK AND POST

    X = vScreen.Width;
    Y = vScreen.Height;
    subDur = size(obj.trace.subTemps,2);
    
    BW = zeros(X, Y);

    for i = 1:subDur
        
        BW(round(subX(i)), round(subY(i))) = BW(round(subX(i)), round(subY(i))) + 1;

    end

    BW(1,:) = 0;
    BW(end,:) = 0;
    BW(:,1) = 0;
    BW(:,end) = 0;

    L = 500;
    S = 50;

    GaussKernel =  fspecial('gaussian', L,S);
    GaussKernel = GaussKernel/max(GaussKernel, [], [1,2]);
    heatMap = conv2(BW, GaussKernel, 'same')/fs;

    clf 

    imshow(frame);
    hold on
    imagesc(heatMap', 'AlphaData', 0.0005*heatMap')
    colormap('hot')
    saveas(gcf,fullfile(savePath,'3_heatmap.png'))
    
    %% 4 - Trajectory and heatmap ONLY FOR TASK AND POST

    clf

    imagesc(frame)
    hold on
    imagesc(heatMap', 'AlphaData', 0.0005*heatMap')
    xticks([])
    yticks([])
       
    colormap('hot');

% 
%     patch(ax2, [subX; flipud(subX)], [subY;flipud(subY)], [1:size(subX,2); fliplr(1:size(subX,2))],...
%         'EdgeColor','interp', 'lineWidth', 1,'MarkerFaceColor','flat');
%     colormap(ax2,'jet');
%     set(ax2, 'YDir', 'reverse')

% 
%     patch(ax2, [subX; flipud(subX)], [subY;flipud(subY)], [1:size(subX,2); fliplr(1:size(subX,2)), 'color', 'k');
%     colormap(ax2,'jet');
%     set(ax2, 'YDir', 'reverse')

    orange = [0.5, 0.25, 0];
    hold on
    plot(obj.sharpTrace.cursor(1,:),obj.sharpTrace.cursor(2,:),'color', orange)

    % set([ax1, ax2], 'Position', [0,0,1,1])

    set(gcf, 'Unit', 'normalized', 'Position', [0,0,1,1])
    
    saveas(gcf,fullfile(savePath, '4_TrajAndHeatmap.png'))
    
    %% 5 - Word Analysis   ONLY FOR TASK AND POST
    
    % Extracting targets 
       BW = frame(:,:,1) <=5;

    ConnectedSets = bwconncomp(BW);
    nbSpots = ConnectedSets.NumObjects;
    a = ConnectedSets.ImageSize;
    Y = a(1);
    X = a(2);

    posPts = zeros(2, nbSpots);
    k = 1;

    for spot = 1:nbSpots

        fprintf('Spot: %d/%d - k=%d\nSize: %d\n\n', spot, nbSpots, k, size(ConnectedSets.PixelIdxList{spot}, 1))

        if size(ConnectedSets.PixelIdxList{spot},1)>= 80

            spotBW = zeros(Y, X);
            spotBW(ConnectedSets.PixelIdxList{spot})=1;
            stats = regionprops(spotBW);

            C1 = stats.Centroid;
            posPts(:,k) = [C1(1), C1(2)];
            k = k+1;

        end
    end

    posPts(:,k:end) = [];
    nbSpots = size(posPts,2);

    fprintf('We found %d targets\n\n', nbSpots)
    
    % Word analysis
    
    spotDist = zeros(nbSpots, subDur);

    for spot = 1:nbSpots

        a = sqrt(sum(([subX(1:subDur)', subY(1:subDur)']-[posPts(:,spot)]').^2,2));
        spotDist(spot,:) = a<40;

    end

    clf

    subplot(2,1,1)

    imagesc(frame)
    axis equal
    xticks([])
    yticks([])
    hold on

    for spot = 1:nbSpots
        text(posPts(1,spot), posPts(2,spot)+80, sprintf('%d', spot),...
            'color', 'red',...
            'HorizontalAlignment', 'center',...
            'FontSize', 5)
    end

    ax2 = subplot(2,1,2)

    imagesc(ax2,spotDist)
    colormap('gray')
    for i = 2:nbSpots
        yline(i-0.5, 'w:')
    end
    ylim([0.5, nbSpots+0.5])
    yticks((1:nbSpots))
    xlabel('Time (s)', 'Interpreter', 'Latex')
    ylabel('Target ID', 'Interpreter', 'Latex')

    set(ax2, 'TickLabelInterpreter', 'Latex')
    
    saveas(gcf,fullfile(savePath, '5_Words.png'));
    
%   %% 6 - Trinary Word Analysis ONLY FOR TASK 
%     
%     xLeft = 605;
%     xRight = 682;
% % 
% %     a = temps>deb;
% %     b = temps<fin;
% %     
% %     ind = find(a+b==2);
% 
%     trinaryWord = zeros(3,subDur);
%     a = subX < xLeft;
%     b = subX > xRight;
%     trinaryWord(1,:) = a(1:subDur);
%     trinaryWord(2,:) = a(1:subDur)+b(1:subDur)==0;
%     trinaryWord(3,:) = b(1:subDur);
%     
%     clf
% 
%     subplot(2,1,1)
% 
%     imagesc(frame)
%     axis equal
%     xticks([])
%     yticks([])
%     hold on
%     xline(xLeft, ':k')
%     xline(xRight, ':k')
% 
%     ax2 = subplot(2,1,2)
%     
%     imagesc(ax2,trinaryWord)
%     colormap('gray')
%     for i = 2:3
%         yline(i-0.5, 'w:')
%     end
%     ylim([0.5, 3+0.5])
%     yticks((1:3))
%     yticklabels({'L', 'O', 'R'})
%     xlabel('Time (s)', 'Interpreter', 'Latex')
%     ylabel('Target ID', 'Interpreter', 'Latex')
% 
%     set(ax2, 'TickLabelInterpreter', 'Latex')
%     
%     saveas(gcf,fullfile(savePath, '6_TrinaryWords.png'));
    
%% 7 - Cluster Word Analysis  ONLY FOR POST

% Extracting mega targets (clustering neighbour targets to extract multi digits targets)

    dists = zeros(3,(nbSpots*(nbSpots-1))/2);

    k = 1;

    % Computing distance between nodes
    
    for spot1 = 1:nbSpots

        for spot2 = 1:spot1-1

            dists(1,k) = spot1;
            dists(2,k) = spot2;
            dists(3,k) = sqrt((posPts(1,spot1)-posPts(1,spot2)).^2+(posPts(2,spot1)-posPts(2,spot2)).^2);
            k = k+1;


        end

    end 

    % computing the clusters of two elements and add them
    
    ind = find(dists(3,:)<55);
    nbClustElt = size(ind,2);
    clustList = zeros(2,nbClustElt);
    clustMem = [];
    
    for clust = 1:nbClustElt
        
        clustList(1,clust) = mean([posPts(1,dists(1, ind(clust))), posPts(1,dists(2, ind(clust)))]);
        clustList(2,clust) = mean([posPts(2,dists(1, ind(clust))), posPts(2,dists(2, ind(clust)))]);
        clustMem = [clustMem, dists(1, ind(clust)), dists(2, ind(clust))];        
        
    end
    
    % Adds single elements cluster
    
    for spot = 1:nbSpots
        
        disp(spot)
        
        if ~ismember(spot, clustMem)
            
            clustList = [clustList, posPts(:,spot)];
            
        end
    end
    
    nbClusters = size(clustList, 2); 
    
  spotDist = zeros(nbClusters, subDur);

    % Reordering Clusters
    
    a = clustList - [X/2; Y/2];
    alphas = atan(a(1,:)./a(2,:));
    [~, Idx] = sort(alphas);
    
    clustList = clustList(:,Idx);

    for spot = 1:nbClusters

        a = sqrt(sum(([subX(1:subDur)', subY(1:subDur)']-[clustList(:,spot)]').^2,2));
        spotDist(spot,:) = a<50;

    end
% a = sqrt(sum(([subX(1:subDur)', subY(1:subDur)']-[posPts(:,spot)]').^2,2));
    clf

    subplot(2,1,1)

    imagesc(frame)
    axis equal
    xticks([])
    yticks([])
    hold on
    viscircles(clustList', 50*ones(1, nbClusters));

    for spot = 1:nbClusters
        text(clustList(1,spot), clustList(2,spot)+80, sprintf('%d', spot),...
            'color', 'blue',...
            'HorizontalAlignment', 'center',...
            'FontSize', 5)
    end

    ax2 = subplot(2,1,2)

    imagesc(ax2,spotDist)
    colormap('gray')
    for i = 2:nbClusters
        yline(i-0.5, 'w:')
    end
    ylim([0.5, nbClusters+0.5])
    yticks((1:nbClusters))
    xlabel('Time (s)', 'Interpreter', 'Latex')
    ylabel('Target ID', 'Interpreter', 'Latex')

    set(ax2, 'TickLabelInterpreter', 'Latex')
    
    %
    
    saveas(gcf,fullfile(savePath, '7_ClusterWords.png'));

   end
    
   
  
end


